<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://qoorts.nl
 * @since      1.0.0
 *
 * @package    Cookiefirst_Plugin
 * @subpackage Cookiefirst_Plugin/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Cookiefirst_Plugin
 * @subpackage Cookiefirst_Plugin/includes
 * @author     Qoorts <rafal@qoorts.nl>
 */
class Cookiefirst_Plugin
{

    /**
     * The loader that's responsible for maintaining and registering all hooks that power
     * the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      Cookiefirst_Plugin_Loader    $loader    Maintains and registers all hooks for the plugin.
     */
    protected $loader;

    /**
     * The unique identifier of this plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $plugin_name    The string used to uniquely identify this plugin.
     */
    protected $plugin_name;

    /**
     * The current version of the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $version    The current version of the plugin.
     */
    protected $version;

    /**
     * Define the core functionality of the plugin.
     *
     * Set the plugin name and the plugin version that can be used throughout the plugin.
     * Load the dependencies, define the locale, and set the hooks for the admin area and
     * the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function __construct()
    {
        if (defined('COOKIEFIRST_PLUGIN_VERSION')) {
            $this->version = COOKIEFIRST_PLUGIN_VERSION;
        } else {
            $this->version = '1.0.0';
        }
        $this->plugin_name = 'cookiefirst-plugin';

        $this->load_dependencies();
        $this->set_locale();
        $this->define_admin_hooks();
        $this->define_public_hooks();

    }

    /**
     * Load the required dependencies for this plugin.
     *
     * Include the following files that make up the plugin:
     *
     * - Cookiefirst_Plugin_Loader. Orchestrates the hooks of the plugin.
     * - Cookiefirst_Plugin_i18n. Defines internationalization functionality.
     * - Cookiefirst_Plugin_Admin. Defines all hooks for the admin area.
     * - Cookiefirst_Plugin_Public. Defines all hooks for the public side of the site.
     *
     * Create an instance of the loader which will be used to register the hooks
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function load_dependencies()
    {

        /**
         * The class responsible for orchestrating the actions and filters of the
         * core plugin.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-cookiefirst-plugin-loader.php';

        /**
         * The class responsible for defining internationalization functionality
         * of the plugin.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-cookiefirst-plugin-i18n.php';

        /**
         * The class responsible for defining all actions that occur in the admin area.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'admin/class-cookiefirst-plugin-admin.php';

        /**
         * The class responsible for defining all actions that occur in the public-facing
         * side of the site.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'public/class-cookiefirst-plugin-public.php';

        $this->loader = new Cookiefirst_Plugin_Loader();

    }

    /**
     * Define the locale for this plugin for internationalization.
     *
     * Uses the Cookiefirst_Plugin_i18n class in order to set the domain and to register the hook
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function set_locale()
    {

        $plugin_i18n = new Cookiefirst_Plugin_i18n();

        $this->loader->add_action('plugins_loaded', $plugin_i18n, 'load_plugin_textdomain');

    }

    /**
     * Register all of the hooks related to the admin area functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_admin_hooks()
    {

        $plugin_admin = new Cookiefirst_Plugin_Admin($this->get_plugin_name(), $this->get_version());

        $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_styles');
        $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts');

        $this->loader->add_action('admin_menu', $this, 'cf_add_admin_menu');
        $this->loader->add_action('admin_init', $this, 'cf_settings_init');

    }

    /**
     * Register all of the hooks related to the public-facing functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_public_hooks()
    {

        $plugin_public = new Cookiefirst_Plugin_Public($this->get_plugin_name(), $this->get_version());

        $this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_styles');
        $this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_scripts');

        add_shortcode('cf_declaration', [$this, 'cf_declaration']);
        $this->loader->add_action('wp_enqueue_scripts', $this, 'cf_footer_script');
        if (function_exists('register_block_type')
        ) {
            $this->loader->add_action('init', $this, 'cf_policy_block');
        }
    }

    /**
     * Run the loader to execute all of the hooks with WordPress.
     *
     * @since    1.0.0
     */
    public function run()
    {
        $this->loader->run();
    }

    /**
     * The name of the plugin used to uniquely identify it within the context of
     * WordPress and to define internationalization functionality.
     *
     * @since     1.0.0
     * @return    string    The name of the plugin.
     */
    public function get_plugin_name()
    {
        return $this->plugin_name;
    }

    /**
     * The reference to the class that orchestrates the hooks with the plugin.
     *
     * @since     1.0.0
     * @return    Cookiefirst_Plugin_Loader    Orchestrates the hooks of the plugin.
     */
    public function get_loader()
    {
        return $this->loader;
    }

    /**
     * Retrieve the version number of the plugin.
     *
     * @since     1.0.0
     * @return    string    The version number of the plugin.
     */
    public function get_version()
    {
        return $this->version;
    }

    /**
     * CookieFirst Policy Gutenberg Block function.
     *
     * @since     1.0.0
     * @return    void    Echoes CookieFirst Policy container
     */
    public function cf_policy_block()
    {
        wp_register_script(
            'cookiefirst-policy',
            plugins_url('block.js', __FILE__),
            array('wp-blocks', 'wp-element')
        );

        register_block_type('cookiefirst/policy', array(
            'editor_script' => 'cookiefirst-policy',
        ));
    }

    /**
     * CookieFirst Policy shortcode function.
     *
     * @since     1.0.0
     * @return    void    Echoes CookieFirst Policy container
     */
    public function cf_declaration($atts = [])
    {
        echo '<div id="cookiefirst-policy-page"></div>';
    }

    /**
     * CookieFirst WP-Admin Setting menu item.
     *
     * @since     1.0.0
     * @return    void    Adds a menu item to wp-admin
     */
    public function cf_add_admin_menu()
    {

        add_submenu_page('options-general.php', 'Cookie First', 'Cookie first', 'manage_options', 'cookiefirst', [$this, 'cf_options_page']);

    }

    /**
     * CookieFirst WP-Admin Setting page.
     *
     * @since     1.0.0
     * @return    void    Initializes the CookieFirst Plugin setting page.
     */
    public function cf_settings_init()
    {

        register_setting('pluginPage', 'cf_settings');

        add_settings_section(
            'cf_pluginPage_section',
            __('', 'cf'),
            [$this, 'cf_settings_section_callback'],
            'cookiefirst'
        );

        add_settings_field(
            'cf_api_key_field',
            __('Your api key', 'cookiefirst'),
            [$this, 'cf_render_api_key_field'],
            'cookiefirst',
            'cf_pluginPage_section'
        );

    }

    /**
     * CookieFirst WP-Admin Setting field.
     *
     * @since     1.0.0
     * @return    void    Renders CookieFirst Plugin ApiKey field.
     */
    public function cf_render_api_key_field()
    {
        $options = get_option('cf_settings');
        echo "<input type='text' name='cf_settings[api_key]' value='" . $options['api_key'] . "'>";
    }

    /**
     * CookieFirst WP-Admin Settings text.
     *
     * @since     1.0.0
     * @return    void    Renders CookieFirst Plugin intro text.
     */
    public function cf_settings_section_callback()
    {

        echo __('Please put in your api key form cookiefirst.com panel', 'cookiefirst');

    }

    /**
     * CookieFirst WP-Admin Setting form.
     *
     * @since     1.0.0
     * @return    void    Renders CookieFirst Plugin Settings Form.
     */
    public function cf_options_page()
    {
        echo "<form action='options.php' method='post'><h1>Cookie First Integration Settings</h1>";
        settings_fields('pluginPage');
        do_settings_sections('cookiefirst');
        submit_button();
        echo '</form>';
    }

    /**
     * CookieFirst WP-Admin footer script.
     *
     * @since     1.0.0
     * @return    void    Inserts CookieFirst script into footer.
     */
    public function cf_footer_script()
    {
        $options = get_option('cf_settings');

        if (array_key_exists('api_key', $options) && !empty($options['api_key'])) {
            wp_enqueue_script(
                'cookiefirst-script',
                sprintf('https://consent.cookiefirst.com/banner.js?cookiefirst-key=%s', $options['api_key'])
            );
        }
    }

}
